/* file_spec.c
 *  Copyright: 2001-2003 The Perl Foundation.  All Rights Reserved.
 *  CVS Info
 *     $Id:
 *  Overview:
 *     File Specification API
 *  Data Structure and Algorithms:
 *  History:
 *  Notes:
 *  References:
 *     A good deal of stuff is borrowed from File::Spec, the Perl5 module
 */


#include "parrot/parrot.h"

/* All filters go here. If we want to add a filter which is
   specific for a OS, it must be macroed with #if PARROT_OS_NAME_IS_ */

ParrotFSFilter Filters[] = {NULL};


/* This is a FS API function. It concatenates parts which could be
   either directory names or a directory name and a file name to form
   a path specific for a given OS */

void
file_spec_cat(struct Parrot_Interp *interpreter,
              STRING *str1, STRING *str2, STRING **dest_str)
{
    STRING *dirsep;
    
    if ((str1 == NULL || string_length(str1) == 0)
        &&
        (str2 == NULL || string_length(str2) == 0))
    {
        *dest_str = string_from_cstring(interpreter, "", 0);
        return; 
    }
    
    dirsep = string_from_cstring(interpreter,
                                 PARROT_FS_DIRSEP,
                                 PARROT_FS_DIRSEP_LEN);
    *dest_str = string_concat(interpreter, str1, dirsep, 1);
    *dest_str = string_append(interpreter, *dest_str, str2, 1);
    
    file_spec_filter(interpreter, dest_str);
    
    return;
}


/* Removes redundant elements like successive slashes on UNIX
   from a path. What is considered to be a redundant element
   is determined by filters that preset the true behavior of
   this function */

void
file_spec_filter(struct Parrot_Interp *interpreter,
                 STRING **str)
{
    int i;
    
    for (i = 0; Filters[i] != NULL; ++ i)
        (*Filters[i])(interpreter, str);
    
    return;
}


 /*
 * Local variables:
 * c-indentation-style: bsd
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
*/